@@ -1,18 +1,18 @@
-{-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE AllowAmbiguousTypes #-}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE DeriveGeneric #-}
-{-# language PatternSynonyms #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE KindSignatures #-}
+{-# LANGUAGE PatternSynonyms #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE TypeApplications #-}
+{-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE UndecidableInstances #-}
-{-# LANGUAGE CPP #-}
 
 module Database.Persist.Class.PersistEntity
     ( PersistEntity (..)
@@ -24,38 +24,47 @@ module Database.Persist.Class.PersistEntity
     , FilterValue (..)
     , BackendSpecificFilter
     , Entity (.., Entity, entityKey, entityVal)
-
+    , ViaPersistEntity (..)
     , recordName
     , entityValues
-    , keyValueEntityToJSON, keyValueEntityFromJSON
-    , entityIdToJSON, entityIdFromJSON
+    , keyValueEntityToJSON
+    , keyValueEntityFromJSON
+    , entityIdToJSON
+    , entityIdFromJSON
+
       -- * PersistField based on other typeclasses
-    , toPersistValueJSON, fromPersistValueJSON
-    , toPersistValueEnum, fromPersistValueEnum
+    , toPersistValueJSON
+    , fromPersistValueJSON
+    , toPersistValueEnum
+    , fromPersistValueEnum
+
       -- * Support for @OverloadedLabels@ with 'EntityField'
     , SymbolToField (..)
-    , -- * Safety check for inserts
-      SafeToInsert
+
+      -- * Safety check for inserts
+    , SafeToInsert
     , SafeToInsertErrorMessage
     ) where
 
+import Data.Functor.Apply (Apply)
 import Data.Functor.Constant
 
 import Data.Aeson
-       ( FromJSON(..)
-       , ToJSON(..)
-       , Value(Object)
-       , fromJSON
-       , object
-       , withObject
-       , (.:)
-       , (.=)
-       )
+    ( FromJSON (..)
+    , ToJSON (..)
+    , Value (Object)
+    , fromJSON
+    , object
+    , withObject
+    , (.:)
+    , (.=)
+    )
 import qualified Data.Aeson.Parser as AP
 import Data.Aeson.Text (encodeToTextBuilder)
-import Data.Aeson.Types (Parser, Result(Error, Success))
+import Data.Aeson.Types (Parser, Result (Error, Success))
 import Data.Attoparsec.ByteString (parseOnly)
 import Data.Functor.Identity
+import Web.PathPieces (PathMultiPiece (..), PathPiece (..))
 
 #if MIN_VERSION_aeson(2,0,0)
 import qualified Data.Aeson.KeyMap as AM
@@ -63,8 +72,8 @@ import qualified Data.Aeson.KeyMap as AM
 import qualified Data.HashMap.Strict as AM
 #endif
 
-import GHC.Records
-import Data.List.NonEmpty (NonEmpty(..))
+import Data.Kind (Type)
+import Data.List.NonEmpty (NonEmpty (..))
 import Data.Maybe (isJust)
 import Data.Text (Text)
 import qualified Data.Text as T
@@ -73,8 +82,8 @@ import qualified Data.Text.Lazy as LT
 import qualified Data.Text.Lazy.Builder as TB
 import GHC.Generics
 import GHC.OverloadedLabels
+import GHC.Records
 import GHC.TypeLits
-import Data.Kind (Type)
 
 import Database.Persist.Class.PersistField
 import Database.Persist.Names
@@ -94,19 +103,30 @@ import Database.Persist.Types.Base
 -- Some advanced type system capabilities are used to make this process
 -- type-safe. Persistent users usually don't need to understand the class
 -- associated data and functions.
-class ( PersistField (Key record), ToJSON (Key record), FromJSON (Key record)
-      , Show (Key record), Read (Key record), Eq (Key record), Ord (Key record))
-  => PersistEntity record where
+class
+    ( PersistField (Key record)
+    , ToJSON (Key record)
+    , FromJSON (Key record)
+    , Show (Key record)
+    , Read (Key record)
+    , Eq (Key record)
+    , Ord (Key record)
+    ) =>
+    PersistEntity record
+    where
     -- | Persistent allows multiple different backends (databases).
     type PersistEntityBackend record
 
     -- | By default, a backend will automatically generate the key
     -- Instead you can specify a Primary key made up of unique values.
     data Key record
+
     -- | A lower-level key operation.
     keyToValues :: Key record -> [PersistValue]
+
     -- | A lower-level key operation.
     keyFromValues :: [PersistValue] -> Either Text (Key record)
+
     -- | A meta-operation to retrieve the 'Key' 'EntityField'.
     persistIdField :: EntityField record (Key record)
 
@@ -120,10 +140,13 @@ class ( PersistField (Key record), ToJSON (Key record), FromJSON (Key record)
     -- language extension to refer to 'EntityField' values polymorphically. See
     -- the documentation on 'SymbolToField' for more information.
     data EntityField record :: Type -> Type
+
     -- | Return meta-data for a given 'EntityField'.
     persistFieldDef :: EntityField record typ -> FieldDef
+
     -- | A meta-operation to get the database fields of a record.
     toPersistFields :: record -> [PersistValue]
+
     -- | A lower-level operation to convert from database values to a Haskell record.
     fromPersistValues :: [PersistValue] -> Either Text record
 
@@ -152,24 +175,41 @@ class ( PersistField (Key record), ToJSON (Key record), FromJSON (Key record)
     --
     -- @since 2.14.0.0
     tabulateEntityA
-        :: Applicative f
+        :: (Applicative f)
         => (forall a. EntityField record a -> f a)
         -- ^ A function that builds a fragment of a record in an
         -- 'Applicative' context.
         -> f (Entity record)
 
+    -- | Like 'tabulateEntityA', but works with any 'Apply' f. This works
+    -- because all entities have at least one field, and so we can tabulate
+    -- things into semigroup-like shapes instead.
+    --
+    -- @since 2.17.0.0
+    tabulateEntityApply
+        :: (Apply f)
+        => (forall a. EntityField record a -> f a)
+        -> f (Entity record)
+
     -- | Unique keys besides the 'Key'.
     data Unique record
+
     -- | A meta operation to retrieve all the 'Unique' keys.
     persistUniqueKeys :: record -> [Unique record]
+
     -- | A lower level operation.
-    persistUniqueToFieldNames :: Unique record -> NonEmpty (FieldNameHS, FieldNameDB)
+    persistUniqueToFieldNames
+        :: Unique record -> NonEmpty (FieldNameHS, FieldNameDB)
+
     -- | A lower level operation.
     persistUniqueToValues :: Unique record -> [PersistValue]
 
     -- | Use a 'PersistField' as a lens.
-    fieldLens :: EntityField record field
-              -> (forall f. Functor f => (field -> f field) -> Entity record -> f (Entity record))
+    fieldLens
+        :: EntityField record field
+        -> ( forall f
+              . (Functor f) => (field -> f field) -> Entity record -> f (Entity record)
+           )
 
     -- | Extract a @'Key' record@ from a @record@ value. Currently, this is
     -- only defined for entities using the @Primary@ syntax for
@@ -180,6 +220,18 @@ class ( PersistField (Key record), ToJSON (Key record), FromJSON (Key record)
     keyFromRecordM :: Maybe (record -> Key record)
     keyFromRecordM = Nothing
 
+-- | Newtype wrapper for optionally deriving typeclass instances on
+-- 'PersistEntity' keys.
+--
+-- @since 2.14.6.0
+newtype ViaPersistEntity record = ViaPersistEntity (Key record)
+
+instance (PersistEntity record) => PathMultiPiece (ViaPersistEntity record) where
+    fromPathMultiPiece pieces = do
+        Right key <- keyFromValues <$> mapM fromPathPiece pieces
+        pure $ ViaPersistEntity key
+    toPathMultiPiece (ViaPersistEntity key) = map toPathPiece $ keyToValues key
+
 -- | Construct an @'Entity' record@ by providing a value for each of the
 -- record's fields.
 --
@@ -213,7 +265,7 @@ class ( PersistField (Key record), ToJSON (Key record), FromJSON (Key record)
 --
 -- @since 2.14.0.0
 tabulateEntity
-    :: PersistEntity record
+    :: (PersistEntity record)
     => (forall a. EntityField record a -> a)
     -> Entity record
 tabulateEntity fromField =
@@ -222,6 +274,7 @@ tabulateEntity fromField =
 type family BackendSpecificUpdate backend record
 
 -- Moved over from Database.Persist.Class.PersistUnique
+
 -- | Textual representation of the record
 recordName
     :: (PersistEntity record)
@@ -231,22 +284,24 @@ recordName = unEntityNameHS . entityHaskell . entityDef . Just
 -- | Updating a database entity.
 --
 -- Persistent users use combinators to create these.
-data Update record = forall typ. PersistField typ => Update
-    { updateField :: EntityField record typ
-    , updateValue :: typ
-    -- FIXME Replace with expr down the road
-    , updateUpdate :: PersistUpdate
-    }
+data Update record
+    = forall typ. (PersistField typ) => Update
+        { updateField :: EntityField record typ
+        , updateValue :: typ
+        , -- FIXME Replace with expr down the road
+          updateUpdate :: PersistUpdate
+        }
     | BackendUpdate
-          (BackendSpecificUpdate (PersistEntityBackend record) record)
+        (BackendSpecificUpdate (PersistEntityBackend record) record)
 
 -- | Query options.
 --
 -- Persistent users use these directly.
-data SelectOpt record = forall typ. Asc  (EntityField record typ)
-                      | forall typ. Desc (EntityField record typ)
-                      | OffsetBy Int
-                      | LimitTo Int
+data SelectOpt record
+    = forall typ. Asc (EntityField record typ)
+    | forall typ. Desc (EntityField record typ)
+    | OffsetBy Int
+    | LimitTo Int
 
 type family BackendSpecificFilter backend record
 
@@ -262,23 +317,25 @@ type family BackendSpecificFilter backend record
 -- 'PersistFilter' requires that you have an array- or list-shaped
 -- 'EntityField'. It is possible to construct values using this that will
 -- create malformed runtime values.
-data Filter record = forall typ. PersistField typ => Filter
-    { filterField  :: EntityField record typ
-    , filterValue  :: FilterValue typ
-    , filterFilter :: PersistFilter -- FIXME
-    }
-    | FilterAnd [Filter record] -- ^ convenient for internal use, not needed for the API
-    | FilterOr  [Filter record]
+data Filter record
+    = forall typ. (PersistField typ) => Filter
+        { filterField :: EntityField record typ
+        , filterValue :: FilterValue typ
+        , filterFilter :: PersistFilter -- FIXME
+        }
+    | -- | convenient for internal use, not needed for the API
+      FilterAnd [Filter record]
+    | FilterOr [Filter record]
     | BackendFilter
-          (BackendSpecificFilter (PersistEntityBackend record) record)
+        (BackendSpecificFilter (PersistEntityBackend record) record)
 
 -- | Value to filter with. Highly dependant on the type of filter used.
 --
 -- @since 2.10.0
 data FilterValue typ where
-  FilterValue  :: typ -> FilterValue typ
-  FilterValues :: [typ] -> FilterValue typ
-  UnsafeValue  :: forall a typ. PersistField a => a -> FilterValue typ
+    FilterValue :: typ -> FilterValue typ
+    FilterValues :: [typ] -> FilterValue typ
+    UnsafeValue :: forall a typ. (PersistField a) => a -> FilterValue typ
 
 -- | Datatype that represents an entity, with both its 'Key' and
 -- its Haskell record representation.
@@ -301,8 +358,8 @@ data FilterValue typ where
 -- migration code -- @persistent@ will expect the column to be in
 -- the middle, but your DBMS will put it as the last column).
 -- So, instead of using a query like the one above, you may use
--- 'Database.Persist.GenericSql.rawSql' (from the
--- "Database.Persist.GenericSql" module) with its /entity
+-- 'Database.Persist.Sql.rawSql' (from the
+-- "Database.Persist.Sql" module) with its /entity
 -- selection placeholder/ (a double question mark @??@).  Using
 -- @rawSql@ the query above must be written as @SELECT ??  WHERE
 -- ..@.  Then @rawSql@ will replace @??@ with the list of all
@@ -310,27 +367,28 @@ data FilterValue typ where
 -- your query returns two entities (i.e. @(Entity backend a,
 -- Entity backend b)@), then you must you use @SELECT ??, ??
 -- WHERE ...@, and so on.
-data Entity record =
-    Entity
-        { entityKey :: Key record
-        , entityVal :: record
-        }
+data Entity record
+    = Entity
+    { entityKey :: Key record
+    , entityVal :: record
+    }
 
-deriving instance (Generic (Key record), Generic record) => Generic (Entity record)
+deriving instance
+    (Generic (Key record), Generic record) => Generic (Entity record)
 deriving instance (Eq (Key record), Eq record) => Eq (Entity record)
 deriving instance (Ord (Key record), Ord record) => Ord (Entity record)
 deriving instance (Show (Key record), Show record) => Show (Entity record)
 deriving instance (Read (Key record), Read record) => Read (Entity record)
 
 -- | Get list of values corresponding to given entity.
-entityValues :: PersistEntity record => Entity record -> [PersistValue]
+entityValues :: (PersistEntity record) => Entity record -> [PersistValue]
 entityValues (Entity k record) =
-  if isJust (entityPrimary ent)
-    then
-      -- TODO: check against the key
-      map toPersistValue (toPersistFields record)
-    else
-      keyToValues k ++ map toPersistValue (toPersistFields record)
+    if isJust (entityPrimary ent)
+        then
+            -- TODO: check against the key
+            map toPersistValue (toPersistFields record)
+        else
+            keyToValues k ++ map toPersistValue (toPersistFields record)
   where
     ent = entityDef $ Just record
 
@@ -343,12 +401,14 @@ entityValues (Entity k record) =
 -- instance ToJSON (Entity User) where
 --     toJSON = keyValueEntityToJSON
 -- @
-keyValueEntityToJSON :: (PersistEntity record, ToJSON record)
-                     => Entity record -> Value
-keyValueEntityToJSON (Entity key value) = object
-    [ "key" .= key
-    , "value" .= value
-    ]
+keyValueEntityToJSON
+    :: (PersistEntity record, ToJSON record)
+    => Entity record -> Value
+keyValueEntityToJSON (Entity key value) =
+    object
+        [ "key" .= key
+        , "value" .= value
+        ]
 
 -- | Predefined @parseJSON@. The input JSON looks like
 -- @{"key": 1, "value": {"name": ...}}@.
@@ -359,11 +419,13 @@ keyValueEntityToJSON (Entity key value) = object
 -- instance FromJSON (Entity User) where
 --     parseJSON = keyValueEntityFromJSON
 -- @
-keyValueEntityFromJSON :: (PersistEntity record, FromJSON record)
-                       => Value -> Parser (Entity record)
-keyValueEntityFromJSON (Object o) = Entity
-    <$> o .: "key"
-    <*> o .: "value"
+keyValueEntityFromJSON
+    :: (PersistEntity record, FromJSON record)
+    => Value -> Parser (Entity record)
+keyValueEntityFromJSON (Object o) =
+    Entity
+        <$> o .: "key"
+        <*> o .: "value"
 keyValueEntityFromJSON _ = fail "keyValueEntityFromJSON: not an object"
 
 -- | Predefined @toJSON@. The resulting JSON looks like
@@ -375,10 +437,11 @@ keyValueEntityFromJSON _ = fail "keyValueEntityFromJSON: not an object"
 -- instance ToJSON (Entity User) where
 --     toJSON = entityIdToJSON
 -- @
-entityIdToJSON :: (PersistEntity record, ToJSON record) => Entity record -> Value
+entityIdToJSON
+    :: (PersistEntity record, ToJSON record) => Entity record -> Value
 entityIdToJSON (Entity key value) = case toJSON value of
-        Object o -> Object $ AM.insert "id" (toJSON key) o
-        x -> x
+    Object o -> Object $ AM.insert "id" (toJSON key) o
+    x -> x
 
 -- | Predefined @parseJSON@. The input JSON looks like
 -- @{"id": 1, "name": ...}@.
@@ -389,7 +452,8 @@ entityIdToJSON (Entity key value) = case toJSON value of
 -- instance FromJSON (Entity User) where
 --     parseJSON = entityIdFromJSON
 -- @
-entityIdFromJSON :: (PersistEntity record, FromJSON record) => Value -> Parser (Entity record)
+entityIdFromJSON
+    :: (PersistEntity record, FromJSON record) => Value -> Parser (Entity record)
 entityIdFromJSON = withObject "entityIdFromJSON" $ \o -> do
     val <- parseJSON (Object o)
     k <- case keyFromRecordM of
@@ -399,24 +463,26 @@ entityIdFromJSON = withObject "entityIdFromJSON" $ \o -> do
             pure $ func val
     pure $ Entity k val
 
-instance (PersistEntity record, PersistField record, PersistField (Key record))
-  => PersistField (Entity record) where
+instance
+    (PersistEntity record, PersistField record, PersistField (Key record))
+    => PersistField (Entity record)
+    where
     toPersistValue (Entity key value) = case toPersistValue value of
         (PersistMap alist) -> PersistMap ((idField, toPersistValue key) : alist)
         _ -> error $ T.unpack $ errMsg "expected PersistMap"
 
     fromPersistValue (PersistMap alist) = case after of
         [] -> Left $ errMsg $ "did not find " `mappend` idField `mappend` " field"
-        ("_id", kv):afterRest ->
+        ("_id", kv) : afterRest ->
             fromPersistValue (PersistMap (before ++ afterRest)) >>= \record ->
                 keyFromValues [kv] >>= \k ->
                     Right (Entity k record)
         _ -> Left $ errMsg $ "impossible id field: " `mappend` T.pack (show alist)
       where
         (before, after) = break ((== idField) . fst) alist
-
-    fromPersistValue x = Left $
-          errMsg "Expected PersistMap, received: " `mappend` T.pack (show x)
+    fromPersistValue x =
+        Left $
+            errMsg "Expected PersistMap, received: " `mappend` T.pack (show x)
 
 errMsg :: Text -> Text
 errMsg = mappend "PersistField entity fromPersistValue: "
@@ -437,7 +503,7 @@ idField = "_id"
 --   fromPersistValue = fromPersistValueJSON
 --   toPersistValue = toPersistValueJSON
 -- @
-toPersistValueJSON :: ToJSON a => a -> PersistValue
+toPersistValueJSON :: (ToJSON a) => a -> PersistValue
 toPersistValueJSON = PersistText . LT.toStrict . TB.toLazyText . encodeToTextBuilder . toJSON
 
 -- | Convenience function for getting a free 'PersistField' instance
@@ -453,20 +519,23 @@ toPersistValueJSON = PersistText . LT.toStrict . TB.toLazyText . encodeToTextBui
 --   fromPersistValue = fromPersistValueJSON
 --   toPersistValue = toPersistValueJSON
 -- @
-fromPersistValueJSON :: FromJSON a => PersistValue -> Either Text a
+fromPersistValueJSON :: (FromJSON a) => PersistValue -> Either Text a
 fromPersistValueJSON z = case z of
-  PersistByteString bs -> mapLeft (T.append "Could not parse the JSON (was a PersistByteString): ")
-                        $ parseGo bs
-  PersistText t -> mapLeft (T.append "Could not parse the JSON (was PersistText): ")
-                 $ parseGo (TE.encodeUtf8 t)
-  a -> Left $ T.append "Expected PersistByteString, received: " (T.pack (show a))
-  where parseGo bs = mapLeft T.pack $ case parseOnly AP.value bs of
-          Left err -> Left err
-          Right v -> case fromJSON v of
+    PersistByteString bs ->
+        mapLeft (T.append "Could not parse the JSON (was a PersistByteString): ") $
+            parseGo bs
+    PersistText t ->
+        mapLeft (T.append "Could not parse the JSON (was PersistText): ") $
+            parseGo (TE.encodeUtf8 t)
+    a -> Left $ T.append "Expected PersistByteString, received: " (T.pack (show a))
+  where
+    parseGo bs = mapLeft T.pack $ case parseOnly AP.value bs of
+        Left err -> Left err
+        Right v -> case fromJSON v of
             Error err -> Left err
             Success a -> Right a
-        mapLeft _ (Right a) = Right a
-        mapLeft f (Left b)  = Left (f b)
+    mapLeft _ (Right a) = Right a
+    mapLeft f (Left b) = Left (f b)
 
 -- | Convenience function for getting a free 'PersistField' instance
 -- from a type with an 'Enum' instance. The function 'derivePersistField'
@@ -484,7 +553,7 @@ fromPersistValueJSON z = case z of
 --   fromPersistValue = fromPersistValueEnum
 --   toPersistValue = toPersistValueEnum
 -- @
-toPersistValueEnum :: Enum a => a -> PersistValue
+toPersistValueEnum :: (Enum a) => a -> PersistValue
 toPersistValueEnum = toPersistValue . fromEnum
 
 -- | Convenience function for getting a free 'PersistField' instance
@@ -502,11 +571,20 @@ toPersistValueEnum = toPersistValue . fromEnum
 -- @
 fromPersistValueEnum :: (Enum a, Bounded a) => PersistValue -> Either Text a
 fromPersistValueEnum v = fromPersistValue v >>= go
-  where go i = let res = toEnum i in
-               if i >= fromEnum (asTypeOf minBound res) && i <= fromEnum (asTypeOf maxBound res)
-                 then Right res
-                 else Left ("The number " `mappend` T.pack (show i) `mappend` " was out of the "
-                  `mappend` "allowed bounds for an enum type")
+  where
+    go i =
+        let
+            res = toEnum i
+         in
+            if i >= fromEnum (asTypeOf minBound res) && i <= fromEnum (asTypeOf maxBound res)
+                then Right res
+                else
+                    Left
+                        ( "The number "
+                            `mappend` T.pack (show i)
+                            `mappend` " was out of the "
+                            `mappend` "allowed bounds for an enum type"
+                        )
 
 -- | This type class is used with the @OverloadedLabels@ extension to
 -- provide a more convenient means of using the 'EntityField' type.
@@ -536,7 +614,7 @@ class SymbolToField (sym :: Symbol) rec typ | sym rec -> typ where
 -- @OverloadedLabels@ support to the 'EntityField' type.
 --
 -- @since 2.11.0.0
-instance SymbolToField sym rec typ => IsLabel sym (EntityField rec typ) where
+instance (SymbolToField sym rec typ) => IsLabel sym (EntityField rec typ) where
     fromLabel = symbolToField @sym
 
 -- | A type class which is used to witness that a type is safe to insert into
@@ -550,25 +628,30 @@ instance SymbolToField sym rec typ => IsLabel sym (EntityField rec typ) where
 -- 'insertKey'.
 --
 -- @since 2.14.0.0
-class SafeToInsert a where
+class SafeToInsert a
 
-type SafeToInsertErrorMessage a
-    = 'Text "The PersistEntity " ':<>: ShowType a ':<>: 'Text " does not have a default primary key."
-    ':$$: 'Text "This means that 'insert' will fail with a database error."
-    ':$$: 'Text "Please  provide a default= clause inthe entity definition,"
-    ':$$: 'Text "or use 'insertKey' instead to provide one."
+type SafeToInsertErrorMessage a =
+    'Text "The PersistEntity "
+        ':<>: ShowType a
+        ':<>: 'Text " does not have a default primary key."
+        ':$$: 'Text "This means that 'insert' will fail with a database error."
+        ':$$: 'Text "Please  provide a default= clause inthe entity definition,"
+        ':$$: 'Text "or use 'insertKey' instead to provide one."
 
 instance (TypeError (FunctionErrorMessage a b)) => SafeToInsert (a -> b)
 
 type FunctionErrorMessage a b =
-    'Text "Uh oh! It looks like you are trying to insert a function into the database."
-    ':$$: 'Text "Argument: " ':<>: 'ShowType a
-    ':$$: 'Text "Result:   " ':<>: 'ShowType b
-    ':$$: 'Text "You probably need to add more arguments to an Entity construction."
+    'Text
+        "Uh oh! It looks like you are trying to insert a function into the database."
+        ':$$: 'Text "Argument: " ':<>: 'ShowType a
+        ':$$: 'Text "Result:   " ':<>: 'ShowType b
+        ':$$: 'Text "You probably need to add more arguments to an Entity construction."
 
 type EntityErrorMessage a =
-    'Text "It looks like you're trying to `insert` an `Entity " ':<>: 'ShowType a ':<>: 'Text "` directly."
-    ':$$: 'Text "You want `insertKey` instead. As an example:"
-    ':$$: 'Text "    insertKey (entityKey ent) (entityVal ent)"
+    'Text "It looks like you're trying to `insert` an `Entity "
+        ':<>: 'ShowType a
+        ':<>: 'Text "` directly."
+        ':$$: 'Text "You want `insertKey` instead. As an example:"
+        ':$$: 'Text "    insertKey (entityKey ent) (entityVal ent)"
 
-instance TypeError (EntityErrorMessage a) => SafeToInsert (Entity a)
+instance (TypeError (EntityErrorMessage a)) => SafeToInsert (Entity a)
