@@ -1,4 +1,11 @@
 {-# LANGUAGE DataKinds #-}
+--
+-- DeriveAnyClass is not actually used by persistent-template
+-- But a long standing bug was that if it was enabled, it was used to derive instead of GeneralizedNewtypeDeriving
+-- This was fixed by using DerivingStrategies to specify newtype deriving should be used.
+-- This pragma is left here as a "test" that deriving works when DeriveAnyClass is enabled.
+-- See https://github.com/yesodweb/persistent/issues/578
+{-# LANGUAGE DeriveAnyClass #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE DerivingStrategies #-}
 {-# LANGUAGE ExistentialQuantification #-}
@@ -14,27 +21,21 @@
 {-# LANGUAGE TypeApplications #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE UndecidableInstances #-}
---
--- DeriveAnyClass is not actually used by persistent-template
--- But a long standing bug was that if it was enabled, it was used to derive instead of GeneralizedNewtypeDeriving
--- This was fixed by using DerivingStrategies to specify newtype deriving should be used.
--- This pragma is left here as a "test" that deriving works when DeriveAnyClass is enabled.
--- See https://github.com/yesodweb/persistent/issues/578
-{-# LANGUAGE DeriveAnyClass #-}
 
 module Database.Persist.THSpec where
 
-import Control.Applicative (Const(..))
+import Control.Applicative (Const (..))
 import Data.Aeson (decode, encode)
 import Data.ByteString.Lazy.Char8 ()
 import Data.Coerce
-import Data.Functor.Identity (Identity(..))
+import Data.Functor.Identity (Identity (..))
 import Data.Int
 import qualified Data.List as List
 import Data.Proxy
 import Data.Text (Text, pack)
 import Data.Time
 import GHC.Generics (Generic)
+import qualified Language.Haskell.TH.Syntax as TH
 import System.Environment
 import Test.Hspec
 import Test.Hspec.QuickCheck
@@ -43,16 +44,18 @@ import Test.QuickCheck.Gen (Gen)
 
 import Database.Persist
 import Database.Persist.EntityDef.Internal
+import Database.Persist.Quasi.Internal (SourceLoc (..), sourceLocFromTHLoc)
 import Database.Persist.Sql
 import Database.Persist.Sql.Util
 import Database.Persist.TH
+import Database.Persist.Types.SourceSpan
 import TemplateTestImports
 
-
 import qualified Database.Persist.TH.CommentSpec as CommentSpec
 import qualified Database.Persist.TH.CompositeKeyStyleSpec as CompositeKeyStyleSpec
 import qualified Database.Persist.TH.DiscoverEntitiesSpec as DiscoverEntitiesSpec
 import qualified Database.Persist.TH.EmbedSpec as EmbedSpec
+import qualified Database.Persist.TH.EntityHaddockSpec as EntityHaddockSpec
 import qualified Database.Persist.TH.ForeignRefSpec as ForeignRefSpec
 import qualified Database.Persist.TH.ImplicitIdColSpec as ImplicitIdColSpec
 import qualified Database.Persist.TH.JsonEncodingSpec as JsonEncodingSpec
@@ -75,7 +78,17 @@ import qualified Database.Persist.TH.TypeLitFieldDefsSpec as TypeLitFieldDefsSpe
 -- machinery
 type TextId = Text
 
-share [mkPersist sqlSettings { mpsGeneric = False, mpsDeriveInstances = [''Generic] }] [persistUpperCase|
+-- | Location above the block defining Person below. Must be before it. Do not move!
+-- Used to test TH definition positions are plausible.
+personDefBeforeLoc :: SourceLoc
+personDefBeforeLoc = $(TH.lift . sourceLocFromTHLoc =<< TH.location)
+
+share
+    [ mkPersistWith
+        sqlSettings{mpsGeneric = False, mpsDeriveInstances = [''Generic]}
+        [entityDef @JsonEncodingSpec.JsonEncoding Proxy]
+    ]
+    [persistUpperCase|
 
 Person json
     name Text
@@ -101,9 +114,20 @@ CustomIdName
     Id      sql=id_col
     name    Text
     deriving Show Eq
+
+QualifiedReference
+    jsonEncoding JsonEncodingSpec.JsonEncodingId
+
 |]
 
-mkPersist sqlSettings [persistLowerCase|
+-- | Location after the block defining Person above. Must be after it. Do not move!
+-- Used to test TH definition positions are plausible.
+personDefAfterLoc :: SourceLoc
+personDefAfterLoc = $(TH.lift . sourceLocFromTHLoc =<< TH.location)
+
+mkPersist
+    sqlSettings
+    [persistLowerCase|
 HasPrimaryDef
     userId Int
     name String
@@ -158,7 +182,9 @@ KeyTable
 
 |]
 
-share [mkPersist sqlSettings { mpsGeneric = False, mpsGenerateLenses = True }] [persistLowerCase|
+share
+    [mkPersist sqlSettings{mpsGeneric = False, mpsGenerateLenses = True}]
+    [persistLowerCase|
 Lperson json
     name Text
     age Int Maybe
@@ -204,9 +230,19 @@ spec = describe "THSpec" $ do
     ToFromPersistValuesSpec.spec
     JsonEncodingSpec.spec
     CommentSpec.spec
+    EntityHaddockSpec.spec
     CompositeKeyStyleSpec.spec
+    it "QualifiedReference" $ do
+        let
+            ed = entityDef @QualifiedReference Proxy
+            [FieldDef{..}] = entityFields ed
+        fieldType `shouldBe` FTTypeCon (Just "JsonEncodingSpec") "JsonEncodingId"
+        fieldSqlType `shouldBe` sqlType @JsonEncodingSpec.JsonEncodingId Proxy
+        fieldReference `shouldBe` ForeignRef (EntityNameHS "JsonEncoding")
+
     describe "TestDefaultKeyCol" $ do
-        let EntityIdField FieldDef{..} =
+        let
+            EntityIdField FieldDef{..} =
                 entityId (entityDef (Proxy @TestDefaultKeyCol))
         it "should be a BackendKey SqlBackend" $ do
             -- the purpose of this test is to verify that a custom Id column of
@@ -216,10 +252,10 @@ spec = describe "THSpec" $ do
             --
             -- should behave like an implicit id column.
             (TestDefaultKeyColKey (SqlBackendKey 32) :: Key TestDefaultKeyCol)
-                `shouldBe`
-                    (toSqlKey 32 :: Key TestDefaultKeyCol)
+                `shouldBe` (toSqlKey 32 :: Key TestDefaultKeyCol)
     describe "HasDefaultId" $ do
-        let EntityIdField FieldDef{..} =
+        let
+            EntityIdField FieldDef{..} =
                 entityId (entityDef (Proxy @HasDefaultId))
         it "should have usual db name" $ do
             fieldDB `shouldBe` FieldNameDB "id"
@@ -233,7 +269,8 @@ spec = describe "THSpec" $ do
             fieldType `shouldBe` FTTypeCon Nothing "HasDefaultIdId"
 
     describe "HasCustomSqlId" $ do
-        let EntityIdField FieldDef{..} =
+        let
+            EntityIdField FieldDef{..} =
                 entityId (entityDef (Proxy @HasCustomSqlId))
         it "should have custom db name" $ do
             fieldDB `shouldBe` FieldNameDB "my_id"
@@ -244,7 +281,8 @@ spec = describe "THSpec" $ do
         it "should have correct haskell type" $ do
             fieldType `shouldBe` FTTypeCon Nothing "String"
     describe "HasIdDef" $ do
-        let EntityIdField FieldDef{..} =
+        let
+            EntityIdField FieldDef{..} =
                 entityId (entityDef (Proxy @HasIdDef))
         it "should have usual db name" $ do
             fieldDB `shouldBe` FieldNameDB "id"
@@ -256,7 +294,8 @@ spec = describe "THSpec" $ do
             fieldType `shouldBe` FTTypeCon Nothing "Int"
 
     describe "SharedPrimaryKey" $ do
-        let sharedDef = entityDef (Proxy @SharedPrimaryKey)
+        let
+            sharedDef = entityDef (Proxy @SharedPrimaryKey)
             EntityIdField FieldDef{..} =
                 entityId sharedDef
         it "should have usual db name" $ do
@@ -271,19 +310,17 @@ spec = describe "THSpec" $ do
             fieldType `shouldBe` (FTTypeCon Nothing "HasDefaultIdId")
         it "should have correct sql type from PersistFieldSql" $ do
             sqlType (Proxy @SharedPrimaryKeyId)
-                `shouldBe`
-                    SqlInt64
+                `shouldBe` SqlInt64
         it "should have same sqlType as underlying record" $ do
             sqlType (Proxy @SharedPrimaryKeyId)
-                `shouldBe`
-                    sqlType (Proxy @HasDefaultIdId)
+                `shouldBe` sqlType (Proxy @HasDefaultIdId)
         it "should be a coercible newtype" $ do
             coerce @Int64 3
-                `shouldBe`
-                    SharedPrimaryKeyKey (toSqlKey 3)
+                `shouldBe` SharedPrimaryKeyKey (toSqlKey 3)
 
     describe "SharedPrimaryKeyWithCascade" $ do
-        let EntityIdField FieldDef{..} =
+        let
+            EntityIdField FieldDef{..} =
                 entityId (entityDef (Proxy @SharedPrimaryKeyWithCascade))
         it "should have usual db name" $ do
             fieldDB `shouldBe` FieldNameDB "id"
@@ -293,17 +330,17 @@ spec = describe "THSpec" $ do
             fieldSqlType `shouldBe` SqlInt64
         it "should have correct haskell type" $ do
             fieldType
-                `shouldBe`
-                    FTApp (FTTypeCon Nothing "Key") (FTTypeCon Nothing "HasDefaultId")
+                `shouldBe` FTApp (FTTypeCon Nothing "Key") (FTTypeCon Nothing "HasDefaultId")
         it "should have cascade in field def" $ do
-            fieldCascade `shouldBe` noCascade { fcOnDelete = Just Cascade }
+            fieldCascade `shouldBe` noCascade{fcOnDelete = Just Cascade}
 
     describe "OnCascadeDelete" $ do
-        let subject :: FieldDef
+        let
+            subject :: FieldDef
             Just subject =
-                List.find ((FieldNameHS "person" ==) . fieldHaskell)
-                $ entityFields
-                $ simpleCascadeDef
+                List.find ((FieldNameHS "person" ==) . fieldHaskell) $
+                    entityFields $
+                        simpleCascadeDef
             simpleCascadeDef =
                 entityDef (Proxy :: Proxy HasSimpleCascadeRef)
             expected =
@@ -312,14 +349,20 @@ spec = describe "THSpec" $ do
                     , fcOnUpdate = Nothing
                     }
         describe "entityDef" $ do
+            it "correct position" $ do
+                let
+                    Just theSpan = entitySpan simpleCascadeDef
+                theSpan `shouldSatisfy` ((> locStartLine personDefBeforeLoc) . spanStartLine)
+                theSpan `shouldSatisfy` (\s -> spanStartLine s < spanEndLine s)
+                theSpan `shouldSatisfy` ((< locStartLine personDefAfterLoc) . spanEndLine)
             it "works" $ do
                 simpleCascadeDef
-                    `shouldBe`
-                        EntityDef
-                            { entityHaskell = EntityNameHS "HasSimpleCascadeRef"
-                            , entityDB = EntityNameDB "HasSimpleCascadeRef"
-                            , entityId =
-                                EntityIdField FieldDef
+                    `shouldBe` EntityDef
+                        { entityHaskell = EntityNameHS "HasSimpleCascadeRef"
+                        , entityDB = EntityNameDB "HasSimpleCascadeRef"
+                        , entityId =
+                            EntityIdField
+                                FieldDef
                                     { fieldHaskell = FieldNameHS "Id"
                                     , fieldDB = FieldNameDB "id"
                                     , fieldType = FTTypeCon Nothing "HasSimpleCascadeRefId"
@@ -333,93 +376,96 @@ spec = describe "THSpec" $ do
                                     , fieldGenerated = Nothing
                                     , fieldIsImplicitIdColumn = True
                                     }
-                            , entityAttrs = []
-                            , entityFields =
-                                [ FieldDef
-                                    { fieldHaskell = FieldNameHS "person"
-                                    , fieldDB = FieldNameDB "person"
-                                    , fieldType = FTTypeCon Nothing "PersonId"
-                                    , fieldSqlType = SqlInt64
-                                    , fieldAttrs = []
-                                    , fieldStrict = True
-                                    , fieldReference =
-                                        ForeignRef
-                                            (EntityNameHS "Person")
-                                    , fieldCascade =
-                                        FieldCascade { fcOnUpdate = Nothing, fcOnDelete = Just Cascade }
-                                    , fieldComments = Nothing
-                                    , fieldGenerated = Nothing
-                                    , fieldIsImplicitIdColumn = False
-                                    }
-                                ]
-                            , entityUniques = []
-                            , entityForeigns = []
-                            , entityDerives =  ["Show", "Eq"]
-                            , entityExtra = mempty
-                            , entitySum = False
-                            , entityComments = Nothing
-                            }
+                        , entityAttrs = []
+                        , entityFields =
+                            [ FieldDef
+                                { fieldHaskell = FieldNameHS "person"
+                                , fieldDB = FieldNameDB "person"
+                                , fieldType = FTTypeCon Nothing "PersonId"
+                                , fieldSqlType = SqlInt64
+                                , fieldAttrs = []
+                                , fieldStrict = True
+                                , fieldReference =
+                                    ForeignRef
+                                        (EntityNameHS "Person")
+                                , fieldCascade =
+                                    FieldCascade{fcOnUpdate = Nothing, fcOnDelete = Just Cascade}
+                                , fieldComments = Nothing
+                                , fieldGenerated = Nothing
+                                , fieldIsImplicitIdColumn = False
+                                }
+                            ]
+                        , entityUniques = []
+                        , entityForeigns = []
+                        , entityDerives = ["Show", "Eq"]
+                        , entityExtra = mempty
+                        , entitySum = False
+                        , entityComments = Nothing
+                        , -- We cannot test this is a precise value without
+                          -- being really fragile, but we have another test to
+                          -- verify the line is in range.
+                          entitySpan = entitySpan simpleCascadeDef
+                        }
         it "has the cascade on the field def" $ do
             fieldCascade subject `shouldBe` expected
         it "doesn't have any extras" $ do
             entityExtra simpleCascadeDef
-                `shouldBe`
-                    mempty
+                `shouldBe` mempty
 
     describe "hasNaturalKey" $ do
-        let subject :: PersistEntity a => Proxy a -> Bool
+        let
+            subject :: (PersistEntity a) => Proxy a -> Bool
             subject p = hasNaturalKey (entityDef p)
         it "is True for Primary keyword" $ do
             subject (Proxy @HasPrimaryDef)
-                `shouldBe`
-                    True
+                `shouldBe` True
         it "is True for multiple Primary columns " $ do
             subject (Proxy @HasMultipleColPrimaryDef)
-                `shouldBe`
-                    True
+                `shouldBe` True
         it "is False for Id keyword" $ do
             subject (Proxy @HasIdDef)
-                `shouldBe`
-                    False
+                `shouldBe` False
         it "is False for unspecified/default id" $ do
             subject (Proxy @HasDefaultId)
-                `shouldBe`
-                    False
+                `shouldBe` False
     describe "hasCompositePrimaryKey" $ do
-        let subject :: PersistEntity a => Proxy a -> Bool
+        let
+            subject :: (PersistEntity a) => Proxy a -> Bool
             subject p = hasCompositePrimaryKey (entityDef p)
         it "is False for Primary with single column" $ do
             subject (Proxy @HasPrimaryDef)
-                `shouldBe`
-                    False
+                `shouldBe` False
         it "is True for multiple Primary columns " $ do
             subject (Proxy @HasMultipleColPrimaryDef)
-                `shouldBe`
-                    True
+                `shouldBe` True
         it "is False for Id keyword" $ do
             subject (Proxy @HasIdDef)
-                `shouldBe`
-                    False
+                `shouldBe` False
         it "is False for unspecified/default id" $ do
             subject (Proxy @HasDefaultId)
-                `shouldBe`
-                    False
+                `shouldBe` False
 
     describe "JSON serialization" $ do
         prop "to/from is idempotent" $ \person ->
             decode (encode person) == Just (person :: Person)
         it "decode" $
-            decode "{\"name\":\"Michael\",\"age\":27,\"foo\":\"Bar\",\"address\":{\"street\":\"Narkis\",\"city\":\"Maalot\"}}" `shouldBe` Just
-                (Person "Michael" (Just 27) Bar $ Address "Narkis" "Maalot" Nothing)
+            decode
+                "{\"name\":\"Michael\",\"age\":27,\"foo\":\"Bar\",\"address\":{\"street\":\"Narkis\",\"city\":\"Maalot\"}}"
+                `shouldBe` Just
+                    (Person "Michael" (Just 27) Bar $ Address "Narkis" "Maalot" Nothing)
     describe "JSON serialization for Entity" $ do
-        let key = PersonKey 0
+        let
+            key = PersonKey 0
         prop "to/from is idempotent" $ \person ->
             decode (encode (Entity key person)) == Just (Entity key (person :: Person))
         it "decode" $
-            decode "{\"id\": 0, \"name\":\"Michael\",\"age\":27,\"foo\":\"Bar\",\"address\":{\"street\":\"Narkis\",\"city\":\"Maalot\"}}" `shouldBe` Just
-                (Entity key (Person "Michael" (Just 27) Bar $ Address "Narkis" "Maalot" Nothing))
+            decode
+                "{\"id\": 0, \"name\":\"Michael\",\"age\":27,\"foo\":\"Bar\",\"address\":{\"street\":\"Narkis\",\"city\":\"Maalot\"}}"
+                `shouldBe` Just
+                    (Entity key (Person "Michael" (Just 27) Bar $ Address "Narkis" "Maalot" Nothing))
     it "lens operations" $ do
-        let street1 = "street1"
+        let
+            street1 = "street1"
             city1 = "city1"
             city2 = "city2"
             zip1 = Just 12345
@@ -434,12 +480,16 @@ spec = describe "THSpec" $ do
         (person1 & ((lpersonAddress . laddressCity) .~ city2)) `shouldBe` person2
     describe "Derived Show/Read instances" $ do
         -- This tests confirms https://github.com/yesodweb/persistent/issues/1104 remains fixed
-        it "includes the name of the newtype when showing/reading a Key, i.e. uses the stock strategy when deriving Show/Read" $ do
-            show (PersonKey 0) `shouldBe` "PersonKey {unPersonKey = SqlBackendKey {unSqlBackendKey = 0}}"
-            read (show (PersonKey 0)) `shouldBe` PersonKey 0
+        it
+            "includes the name of the newtype when showing/reading a Key, i.e. uses the stock strategy when deriving Show/Read"
+            $ do
+                show (PersonKey 0)
+                    `shouldBe` "PersonKey {unPersonKey = SqlBackendKey {unSqlBackendKey = 0}}"
+                read (show (PersonKey 0)) `shouldBe` PersonKey 0
 
-            show (CustomPrimaryKeyKey 0) `shouldBe` "CustomPrimaryKeyKey {unCustomPrimaryKeyKey = 0}"
-            read (show (CustomPrimaryKeyKey 0)) `shouldBe` CustomPrimaryKeyKey 0
+                show (CustomPrimaryKeyKey 0)
+                    `shouldBe` "CustomPrimaryKeyKey {unCustomPrimaryKeyKey = 0}"
+                read (show (CustomPrimaryKeyKey 0)) `shouldBe` CustomPrimaryKeyKey 0
 
     describe "tabulateEntityA" $ do
         it "works" $ do
@@ -454,20 +504,25 @@ spec = describe "THSpec" $ do
                         _ <- lookupEnv "PERSON_FOO" :: IO (Maybe String)
                         pure Bar
                     PersonAddress ->
-                        pure $ Address  "lol no" "Denver" Nothing
+                        pure $ Address "lol no" "Denver" Nothing
                     PersonId ->
                         pure $ toSqlKey 123
-            expectedAge <- fromInteger . subtract 1988 . (\(a, _, _) -> a) . toGregorian . utctDay <$> getCurrentTime
-            person `shouldBe` Entity (toSqlKey 123) Person
-                { personName =
-                    "Matt"
-                , personAge =
-                    Just expectedAge
-                , personFoo =
-                    Bar
-                , personAddress =
-                    Address  "lol no" "Denver" Nothing
-                }
+            expectedAge <-
+                fromInteger . subtract 1988 . (\(a, _, _) -> a) . toGregorian . utctDay
+                    <$> getCurrentTime
+            person
+                `shouldBe` Entity
+                    (toSqlKey 123)
+                    Person
+                        { personName =
+                            "Matt"
+                        , personAge =
+                            Just expectedAge
+                        , personFoo =
+                            Bar
+                        , personAddress =
+                            Address "lol no" "Denver" Nothing
+                        }
 
     describe "tabulateEntity" $ do
         it "works" $ do
@@ -482,28 +537,33 @@ spec = describe "THSpec" $ do
                             "Denver"
                         AddressZip ->
                             Nothing
-            addressTabulate `shouldBe`
-                Entity (toSqlKey 123) Address
-                    { addressStreet = "nope"
-                    , addressCity = "Denver"
-                    , addressZip = Nothing
-                    }
+            addressTabulate
+                `shouldBe` Entity
+                    (toSqlKey 123)
+                    Address
+                        { addressStreet = "nope"
+                        , addressCity = "Denver"
+                        , addressZip = Nothing
+                        }
 
     describe "CustomIdName" $ do
         it "has a good safe to insert class instance" $ do
-            let proxy = Proxy :: SafeToInsert CustomIdName => Proxy CustomIdName
+            let
+                proxy = Proxy :: (SafeToInsert CustomIdName) => Proxy CustomIdName
             proxy `shouldBe` Proxy
 
 (&) :: a -> (a -> b) -> b
 x & f = f x
 
-(^.) :: s
-     -> ((a -> Const a b) -> (s -> Const a t))
-     -> a
+(^.)
+    :: s
+    -> ((a -> Const a b) -> (s -> Const a t))
+    -> a
 x ^. lens = getConst $ lens Const x
 
-(.~) :: ((a -> Identity b) -> (s -> Identity t))
-     -> b
-     -> s
-     -> t
+(.~)
+    :: ((a -> Identity b) -> (s -> Identity t))
+    -> b
+    -> s
+    -> t
 lens .~ val = runIdentity . lens (\_ -> Identity val)
